//
//  MCForegroundProcessMonitor.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 3/16/05.
//  Copyright 2005 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCForegroundProcessMonitor.h"

NSString* MCForegroundProcessChanged = @"MCForegroundProcessChanged";

@interface MCForegroundProcessMonitor(Private)
- (void)_foregroundProcessChanged: (id)sender;
pascal OSErr _appFrontSwitched(EventHandlerCallRef inHandlerRef, EventRef inEvent, void *inUserData);
@end

@implementation MCForegroundProcessMonitor

static MCForegroundProcessMonitor* _sharedWatcher = nil;

+ (MCForegroundProcessMonitor*)sharedWatcher
{
	if( _sharedWatcher == nil )
	{
		//We do this in two-steps incase -init calls +sharedWatcher
		_sharedWatcher = [MCForegroundProcessMonitor alloc];
		_sharedWatcher = [_sharedWatcher init];
	}
	
	return _sharedWatcher;
}

+ (void)terminateSharedWatcher
{
	if( _sharedWatcher )
	{
		[_sharedWatcher release];
		_sharedWatcher = nil;
	}
}

- (void)_initFor103
{
	/* If we are on 10.3 or earlier, we can use NSDistributedNotificationCenter to get notifications */
	[[NSDistributedNotificationCenter defaultCenter]
		addObserver:self selector:@selector(_foregroundProcessChanged:) name:@"com.apple.HIToolbox.menuBarShownNotification" object:nil];
}

- (void)_initFor104
{
	OSStatus err;
	EventTypeSpec eventSpec = { kEventClassApplication, kEventAppFrontSwitched };


	/* If we are on 10.4, we have to use Carbon Events */

	err = InstallEventHandler( GetApplicationEventTarget(), (EventHandlerProcPtr)_appFrontSwitched, 1, &eventSpec, self, &mEventHnadler);

	if( err )
	{
		mEventHnadler = nil;
	}

	[[[NSWorkspace sharedWorkspace] notificationCenter]
		addObserver:self selector:@selector(_foregroundProcessChanged:) name:NSWorkspaceDidLaunchApplicationNotification object:nil];
	[[[NSWorkspace sharedWorkspace] notificationCenter]
		addObserver:self selector:@selector(_foregroundProcessChanged:) name:NSWorkspaceDidTerminateApplicationNotification object:nil];
}

- (id)init
{
	self = [super init];
	
	if( self )
	{
		mIsOn103 = ![NSMenu instancesRespondToSelector: @selector(menuBarHeight)];
				
		if( mIsOn103 )
			[self _initFor103];
		else
			[self _initFor104];
	}
	
	return self;
}

- (void)dealloc
{
	if( self == _sharedWatcher )
		_sharedWatcher = nil;

	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver: self];

	if( mIsOn103 )
	{
		[[NSDistributedNotificationCenter defaultCenter] removeObserver: self];
	}
	else
	{
		if( mEventHnadler )
		{
			RemoveEventHandler( mEventHnadler );
			mEventHnadler = nil;
		}
	}

	[super dealloc];
}

- (void)_foregroundProcessChanged: (id)sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName: MCForegroundProcessChanged object: self];
}

pascal OSErr _appFrontSwitched(EventHandlerCallRef inHandlerRef, EventRef inEvent, void *inUserData)
{
	[_sharedWatcher _foregroundProcessChanged: nil];
	return noErr;
}

@end

