//
//  MCMenuExtraView.m
//  MemoryCell
//
//  Created by Quentin Carnicelli on 11/29/04.
//  Copyright 2004 Rogue Amoeba Software, LLC. All rights reserved.
//

#import "MCMenuExtraView.h"

#include <Carbon/Carbon.h>

@implementation MCMenuExtraView

+ (NSImage*)_loadImageNamed: (NSString*)name
{
	NSString *path;
	NSImage* image;

	path = [[NSBundle bundleForClass: self] pathForImageResource:name];
	if( path )
		image = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];

	return image;
}

+ (NSImage*)chipImage: (BOOL)highlighted
{
	static NSImage* bgImage = nil;
	static NSImage* bgImageAlt = nil;
	NSImage* image;
	
	if( highlighted )
	{
		if( bgImageAlt == nil )
			bgImageAlt = [[self _loadImageNamed: @"menuIconAlt"] retain];
		image = bgImageAlt;
	}
	else
	{
		if( bgImage == nil )
			bgImage = [[self _loadImageNamed: @"menuIcon"] retain];
		image = bgImage;
	}
	
	return image;
}

+ (NSSize)contentSize
{
	NSSize size;
	size.height = [[NSStatusBar systemStatusBar] thickness];
	size.width = 63;
	return size;
}

- (void)setUnit: (process_memory_stat_unit)unit
{
	if( mDisplayUnit != unit )
	{
		mDisplayUnit = unit;
		[self setNeedsDisplay: YES];
	}
}

- (process_memory_stat_unit)unit
{
	return mDisplayUnit;
}

- (void)setProcess: (const process_memory_stats*)process
{
	pid_t lastPid;
	BOOL needsRedraw;

	lastPid = mProcessStats.fPid;
	
	if( !process )
		memset( &mProcessStats, 0, sizeof(mProcessStats) );
	else
		memcpy( &mProcessStats, process, sizeof(mProcessStats) );
		
	needsRedraw = (mProcessStats.fPid != lastPid) ||
				  (mProcessStats.fTimestamp != mLastDrawnTimestamp);

	[self setNeedsDisplay: needsRedraw];
}

- (const process_memory_stats*)process
{
	return &mProcessStats;
}

#pragma mark -

- (NSString*)_displayStringForUnits: (process_memory_stat_unit)memUnit
{
	float size;
	long change;
	NSString* unit;
	NSString* frmt;
	NSString* text;
	NSString* suffix;
	
	size = (float)process_mem_stats_get_value( &mProcessStats, memUnit );
	size = size / 1024;
	if( size < 1000 )
	{
		unit = @"K";
	}
	else
	{
		size /= 1024.0;
		if( size < 1000 )
		{
			unit = @"M";
		}
		else
		{
			size /= 1024.0;
			unit = @"G";
		}
	}

	if( size < 100 )
		frmt = @"%04.1f";
	else
		frmt = @"%.0f";

	text = [NSString stringWithFormat: frmt, size];
	
	change = process_mem_stats_get_value_change( &mProcessStats, memUnit );
	if( abs( change ) < (50*1024) )
		suffix = unit;
	else if( change < 0 )
		suffix = @"-";
	else
		suffix = @"+";

	text = [NSString stringWithFormat: @"(%@%@)", text, suffix];

	return text;
}

- (NSColor*)_menuTextColor: (BOOL)highlighted error: (BOOL)hasError
{
	OSStatus err=noErr;
	RGBColor rgbColor;
	
	//if( !highlighted && hasError )
	//	return [NSColor redColor];
	
	err = GetThemeTextColor(highlighted ? kThemeTextColorRootMenuSelected :kThemeTextColorRootMenuActive, 32, TRUE, &rgbColor);
	if( !err )
	{
		float red = (float)rgbColor.red / (float)0xFFFF;
		float green = (float)rgbColor.green / (float)0xFFFF;
		float blue = (float)rgbColor.blue / (float)0xFFFF;
		float alpha = 1.0;
		return [NSColor colorWithDeviceRed: red green: green blue: blue alpha: alpha];
	}
	
	return highlighted ? [NSColor whiteColor] : [NSColor blackColor];
}

- (NSMutableAttributedString*)_displayAttributedStringForUnits: (process_memory_stat_unit)memUnit highlighted: (BOOL)highlighted
{
	NSMutableAttributedString* attrStr;
	id val;
	NSRange entire;

	if( mProcessStats.fStatus == kProcessMemoryStatsError || !mProcessStats.fPid )
		val = @"(Error)";
	else if( mProcessStats.fStatus == kProcessMemoryStatsNoData )
		val = @"(N/A)";
	else
		val = [self _displayStringForUnits: memUnit];

	entire = NSMakeRange( 0, [(NSString*)val length] );
	attrStr = [[NSMutableAttributedString alloc] initWithString: val];
	
	val = [NSFont systemFontOfSize: 12.0f];
	[attrStr addAttribute: NSFontAttributeName value: val range: entire];

	val = [self _menuTextColor: highlighted error: (mProcessStats.fPid==0)];
	[attrStr addAttribute: NSForegroundColorAttributeName value: val range: entire];

	[attrStr setAlignment: NSLeftTextAlignment range: entire];

	return [attrStr autorelease];
}

- (void)drawRect:(NSRect)frame
{
	BOOL highlighted;
	NSRect rect;
	NSAttributedString* attrStr;

	highlighted = [_menuExtra isMenuDown];

	[[[self class] chipImage: highlighted] compositeToPoint: NSMakePoint(0, 4) operation: NSCompositeSourceOver];
	
	rect = frame;
	rect.origin.x += 17;
	rect.size.width -= 18;
	rect.origin.y -= 2;
	rect.origin.y += .5;
	rect.origin.x += .5;
	attrStr = [self _displayAttributedStringForUnits: mDisplayUnit highlighted: highlighted];
	[attrStr drawInRect: rect];
	
	mLastDrawnTimestamp = mProcessStats.fTimestamp;
}

#pragma mark -

- (NSString*)AXValue
{
	float size;
	NSString* unit;
	NSString* frmt;
	NSString* text;
	NSString* suffix;

	if( !mProcessStats.fPid || !mDisplayUnit )
		return @"";

	size = (float)process_mem_stats_get_value( &mProcessStats, mDisplayUnit );
	size = size / 1024;
	if( size < 1024 )
		unit = @"kilobytes";
	else
	{
		size /= 1024;
		unit = @"megabytes";
	}

	if( size < 100 )
		frmt = @"%.1f %@";
	else
		frmt = @"%.0f %@";

	text = [NSString stringWithFormat: frmt, size, unit];

	return text;
}

@end
